class Category < ActiveRecord::Base
	has_and_belongs_to_many :games

	#TODm blank in-between forbidden
	validates	:name,			length: {	:maximum => 20 },
								format: {	:with => /\A[^<>&: ]+( [^<>&: ]+)*\z/, 
										    :message => "HTML-tag-related, :, and & characters are not allowed" }

	#TODm This validation is "prone" to double blank difference error:
	validates_uniqueness_of :name, scope: [:parent_id]

	validates	:description,	format: {	:with => /\A[^<>&"]{0,400}\z/, 
											:message => "No html markup. Not more than 400 characters are allowed" }
	validates	:discussion,	length: {	:minimum => 0, :maximum => 2500 }
	validate	:no_orphans

	ALL = '_all'
	@@cats = nil

	def self.cats=(c)
		@@cats=c
	end


	#=======================================================================
	# master cats generator
	# builds @@cats object; see comments to it in this method below
	#-----------------------------------------------------------------------
	def self.cats
		return @@cats if @@cats

		#if we cannot get right relations right from Category and db,
		#let's just sort them as array:
		catobs=self.all
		catobs.sort! do |a,b|
			a.name <=> b.name
		end

		hcats={} #cats indexed by path
		catx={}  #cats indexed by id

		topwrap={:_ => {}}	#to enable Category existance for migration when no population
		topchildren={ ALL.to_sym => topwrap }

		#supplies parent-of-ALL for ALL category
		superwrap={ :_ => topchildren}
		catx[-1]=superwrap


		#build array catx which maps index to cats:
		catobs.each do |model|
			if model.name == ALL
				topwrap[:model]=model
				w=topwrap
			else
				w={ :_ => {}, :model => model }
			end
			catx[model.id]=w
		end


		#make a tree rooted in _all category
		catx.each do |key,wrap|
			next if wrap.object_id == superwrap.object_id
			model=wrap[:model]
			cparent_id = model.parent_id
			if cparent_id && cparent_id > -1
				#give the parent its named children:
				catx[cparent_id][:_][model.name.to_sym]=wrap
			end
		end

		self.constructing_paths(topchildren, '')
		#now, our nodes have own full paths like _all:moo:boo...

		#build a hash: {full-path => cat, ... }:
		catx.each do |index,wrap|
			hcats[wrap[:path]]=wrap
		end


 		@@cats={	:hash => hcats,
					:indexed => catx,
					ALL.to_sym => topchildren,	#for admin paths; includes root-category-name into path
					:_ => topwrap[:_]			#sugar for short paths; excludes root-category-name from path
		}
		self.find_cats_population
		self.make_path_and_options
		@@cats
	end


	def self.constructing_paths(children,path)
		prefix= path + (path.empty? ? '' : ':')
		if children && !children.empty?
			children.each do |k,wrap|
				new_path=prefix + k.to_s
				wrap[:path]=new_path
				self.constructing_paths(wrap[:_], new_path)
			end
		end
	end

	def self.find_cats_population
		#TODm waste of resources?:
		@@cats[:indexed].each do |index, wrap|
			next if index == -1
			model=wrap[:model]
			k=model.name
			population=model.games #too many queries TODm do like: all_games=Game.all(:sort => 'ASC') ... no inner join with cats..games, grop by cat and then assign
			if population and population.length > 0
				#same problem ... we want to have children sorted ... so
				#do this in ugly way ... manually
				population.sort! do |a,b|
					a.name <=> b.name
				end
				wrap[:population]=population
			end
		end
		#puts "find_cats_population completed"
	end

	#create paths and select-el-options
	def self.make_path_and_options()
		self.cats if !@@cats
		return @@cats[:options] if @@cats[:options]  #don't calculate twice
		@@cats[:options]=cats2options_generic(@@cats[ALL.to_sym], '','',0)
	end
	#---------------------------------------------------
	# master cats generator
	#===================================================




	#===================================================
	# additional helpers called only when requested:
	#===================================================
	#Returns:	cats converted to table's rows, not wrapped into <table>:
	#Input:		-2 - show approved
	#			-1 - show all
	#			real-user-id - show approved and belonging to this user
	def self.cats2table(ob,res, ind, user_id=-1)
		self.cats if !@@cats
		if ob && !ob.empty?
			ob.each do |k,wrap|
				model=wrap[:model]
				if user_id != -1
					next if model.user_id != user_id and !model.approved
				end

				indent ='<tr>'+ind	
				res << indent

				#add <anchor> to populated cats
				#this title overrides category tooltip: better put like (6) ...
				anchor1 = wrap[:population] ?	"<a title=\""+wrap[:population].length.to_s+" games to preview ... \" "+
											"href=\"/search_games?category_id=#{model.id}\">" : ''
				anchor2 = wrap[:population] ? "</a>" : ''

				#add cat's tooltip:
				res << '<td title="'+
						
						#for case of web-unsafe: model.description.to_s.dohtmlesc(model.approved)+ #escape non-approved description
						model.description.to_s+

						'" >'+anchor1 + k.to_s+anchor2+

						#to add more content for specific category later in view:
						"<!-- substitution key #{model.id} -->" + 

						'</td></tr>'+"\n"
				res = cats2table(wrap[:_],res, ind+'<td></td>', user_id)
			end
		end
		res
	end

	def self.cats2htmltable(user_id=-1)
		self.cats if !@@cats
		res='<table>'+
			cats2table(@@cats[:_], '','', user_id)+
			'</table>'
	end




	#Returns: cats converted to select form's element
	def self.cats2select(param_name,guest=false, id='')
		id =" id=\"#{id}\" " if id != ''
		res="\t\t<select #{id} name=\"#{param_name}\" >"+
			"\t\t\t<option value=\"\" ></option>\n" #non-selected
		if guest 
			res << self.make_options_for_guest
		else
			res << self.make_path_and_options
		end
		res << "</select>\n\n"
	end






	#=========================================
	#guest
	#-----------------------------------------
	#create paths and select-el-options
	def self.make_options_for_guest()
		self.cats if !@@cats
		return @@cats[:options_for_guest] if @@cats[:options_for_guest]
		@@cats[:options_for_guest]=self.cats2options_generic(@@cats[:_], '','',0,'only linkable')
	end

	#only non-top-cat for guest yet: TODm possibly not used now:
	def self.cats2options_generic(ob,res, path, level,linkable_only=false)
		prefix= path + (path.empty? ? '' : ':')
		if ob && !ob.empty?
			ob.each do |k,wrap|
				model=wrap[:model]
				break if !model #TODm do better missed migration protection 
				new_path=prefix + k.to_s
				if !linkable_only || model.linkable
					res << "\t\t\t<option value=\"#{model.id}\">#{new_path}</option>\n"
					res = self.cats2options_generic(wrap[:_],res,new_path, level+1, linkable_only)
				end
			end
		end
		res
	end
	#-----------------------------------------
	#guest
	#=========================================

private

	def no_orphans
	    if	!parent_id or (parent_id < 0 and name != ALL)
	      	errors.add(:parent_id, "Parent can be missed or negative only for the "+ALL+" category")
		end
	end
	
end
