=begin #************************************************************

Purpose:	generate models, routes, scaffols
			from settings and table's metaschema-text
Produces:   @scaffolds
			@routes
			@models
Required:	build ROR-style app-folder-tree to be able do "save"
Input:		configurations defined in first GenerateMVC class opening:
				@tables
				@link_tables
Internal:	@parsed_tables
			@routes_resources
			...		

TODm:		replace name getters with stubs like:
					cchildren	=@models[cchild][:plural_camel]

TODm?		:dependent => :destroy when <has one> macro exists

=end #**************************************************************

require File.expand_path('../tp/string.rb', __FILE__)
require File.expand_path('../tp/dig.rb', __FILE__)
require File.expand_path('../tp/file.rb', __FILE__)
require 'fileutils.rb';

class GenerateMVC

  def self.build_all(application_name,login=true)
	raise 'missed applicaiton name' if !application_name
	@do_login=login
	self.parsed_tables #relations extracted here
	self.build_view_shows
	self.parse_linking_tables
	self.add_relations_to_route_resources
	self.finish_routes_resources
	self.build_routes(application_name, login)
	self.build_migrations
	self.build_model_definitions
	self.build_relational_hforms_and_shows
	self.build_model_controllers
	self.build_shared_footnotes
	self.build_login if @do_login
  end

  def self.save_all(root_to_save='.')
	r=root_to_save
	self.save_models r
	self.save_relational_hforms_and_shows r
	self.save_model_controllers r
	self.save_view_shows r
  	self.save_shared_footnotes r
	self.save_login(r) if @do_login
    self.save_routes(r)
	FileLib.save r+'/db/migrate/001_create_my_tables.rb', self.build_migrations
  end


  #============================================================
  #Output: array of parsed-lines
  #        parsed-line = array of tokens separated by spacers
  #        all tokens and lines are trimmed from spacers
  #============================================================
  def self.parse_lines( t )
    tt = t.split( "\n" )
    result = []
    tt.each do |v|
      v.strip!
      next if v == ''
      r = v.split(/(\s)+/)
      w = []
      r.each do |v|
        w << v if v.strip != ''
      end
      result << w if w.size > 0
    end
    return result
  end



  #************************************************************
  #Preliminary handling
  #============================================================

  #////////////////////////////////////////////////////////////
  #Purpose	do checks and parse @tables
  #Input	table metashema-text
  #Output	@parsed_tables
  #			@models	
  #			@models_decamelized
  #============================================================
  def self.parsed_tables
	return @parsed_tables if @parsed_tables
    @parsed_tables = @tables.split('&').collect {|t| self.parse_lines(t) }

	#fix plural name
	@parsed_tables.each do |t|
		t.each_with_index do |line,i|
			if i==0 
				if line.size == 1
					#no plural name is submitted, generate default by adding "s":
					line << line[0].decamelize+'s'
				end
			end
			raise "Missed #{i-1} field name in table #{t[0][0]}"if line.size < 2
		end
	end


	#check field type
	@parsed_tables.each do |t|
		t.each_with_index do |line,i|
			if i>0 
				case line[0]
					when 'string'
						next
					when 'integer'
						next
					when 'boolean'
						next
					when 'text'
						next
					when 'datetime'
						next
				end
				raise "Unspecifield field type \"#{line[0]}\" in line #{i} in table #{t[0][0]}"
			end
		end
	end


	#organize models
	@models={}
	@models_decamelized={}
	@parsed_tables.each do |t|
		name=t[0][0]
		decamelized = name.decamelize
		plural=t[0][1]
		model=	{	:table => t, :name => name, :decamelized => decamelized,
					:plural => plural, :plural_camel=>plural.camelize
				}
		@models[name]=model
		@models_decamelized[decamelized]=model
	end	

	#Do a check of @link_tables input:
	#Will raise an error if link_table has a wrong name:
    @link_tables.each do |ltab|
       first_table_name=ltab[0]
       second_table_name=ltab[1]
		self.find_table_by_name first_table_name
		self.find_table_by_name second_table_name
	end



	#========================================================================================
	#find <relations> in meta-table lines, collect them, remove from field definitions, and
	#----------------------------------------------------------------------------------------
	#accumulate in collection @relations ready for builing model definitions, router resources,
	#and view/"_show" files ...
	@relations={}
	@models.each do |name,model| # name is a "child": name -> tname ("parent")
		t=model[:table]
		t.each_with_index do |line,i|
			next if i==0
			field_name=line[1]
			replaced = -1
			line.each_with_index do |token,i|
				next if i== 0
				#extract relation and replace it with  ''
				token.gsub!(/<([^<]*)>/) do |s|
					#check format xxxx_id:
					if field_name.slice(-3,3)=='_id'
						decam_tname=field_name.slice(0, field_name.size-3)

						#we found "parent":
						target=find_table_by_name(decam_tname,'decamelized')

						tname=target[0][0]
						@relations[tname] ||= {} 
						w=@relations[tname][name] = {} 
						w[:type]=$1
						if $1 == 'has_many'
							w[:for_model]=$1 + " :#{t[0][1]}" + ", dependent: :destroy "
							#@relations[tname] << $1 + " :#{t[0][1]}"
						elsif $1 == 'has_one'
							w[:for_model]=$1 + " :#{name.decamelize}" + ", dependent: :destroy "
						end
						#good: @relations[tname].dig "at construction for parent #{tname}: "
					end
					replaced=i
					'' #replace with '' #must remove not replace.
				end #token
			end #line.each
			if replaced > -1
				#must remove not replace ... removing
				#puts 'replaced. index=' + replaced.to_s + ' ' + line[0]
				line.slice!(replaced)
			end
		end #t.each
	end
	#----------------------------------------------------------------------------------------
	#find <relations> in meta-table lines, collect them, remove from field definitions, and
	#========================================================================================




	return @parsed_tables
  end #def self.parsed_tables
  #////////////////////////////////////////////////////////////
  #Preliminary handling
  #************************************************************







  #Looks up table in paresed tables by name
  #Checks existance of a table.
  def self.find_table_by_name(name, decamelized=false)
	if decamelized
		return @models_decamelized[name][:table] if @models_decamelized[name]
	else
		return @models[name][:table] if @models[name]
	end
	raise "Table \"#{name}\" is not found ... "
  end




  #==============================================================
  # prebuilds	linking_tables, 
  #				@has_and_belongs_to_many
  #				@routes_resources
  #==============================================================
  def self.parse_linking_tables
	return @linking_tables if @linking_tables
	pt=self.parsed_tables    
	t =<<-TABLE
            create_table '<=>plural1<=>_<=>plural2<=>', :id => false do |t|
              t.integer  '<=>singular1<=>_id',            :default => 0,     :null => false
              t.integer  '<=>singular2<=>_id',            :default => 0,     :null => false
            end

    TABLE

	@has_and_belongs_to_many ||={}
	h=@has_and_belongs_to_many;
	@routes_resources ||={}
	routes=@routes_resources

    lt = @link_tables
    r  = ''
    dropper = ''
    ind="            "
    lt.each do |ltab|

       first_table_name=ltab[0]
       second_table_name=ltab[1]
		
		#find first table
		f=-1
		s=-1
		pt.each_with_index do |t,i|
			f=i if t[0][0]==first_table_name
			s=i if t[0][0]==second_table_name
		end

		#Parser sugar:
		raise "table #{first_table_name} missed and cannot be joined ..." if f <0
		raise "table #{second_table_name} missed and cannot be joined ..." if s <0


       singular1 = pt[f][0][0]
       plural1   = pt[f][0][1]
       singular2 = pt[s][0][0]
       plural2   = pt[s][0][1]

		w="\thas_and_belongs_to_many :"+plural2 + "\n"
		h[singular1] = h[singular1] ? h[singular1] << w :  w
		w="\thas_and_belongs_to_many :"+plural1 + "\n"
		h[singular2] = h[singular2] ? h[singular2] << w :  w

		routes[singular1] ||= ''
		routes[singular1] += "\t\tresources :"+plural2 + "\n"
		routes[singular2] ||= ''
		routes[singular2] += "\t\tresources :"+plural1 + "\n"

	
		if plural2 < plural1
			#puts "swapping linked tables to comply \"lexicographical\" order"
			w=singular2
			singular2=singular1
			singular1=w
			w=plural2
			plural2=plural1
			plural1=w
		end

       tt = t
       tt = tt.gsub( '<=>plural1<=>', plural1 )
       tt.gsub!( '<=>plural2<=>', plural2 )
       tt.gsub!( '<=>singular1<=>', singular1.decamelize )
       tt.gsub!( '<=>singular2<=>', singular2.decamelize )
       r += tt
       dropper += ind + 'drop_table :' + plural1 + '_' + plural2 + "\n"
    end

	#vital debug
	#@has_and_belongs_to_many.digm 'lt: @has_and_belongs_to_many'
	#@routes_resources.digm 'lt: @routes_resources'
	@linking_tables = [r,dropper]
    return @linking_tables
  end





  #==============================================================
  # migrations
  #--------------------------------------------------------------
  def self.build_migrations
	return @migrations if @migrations 
    r=<<-END
       class CreateMyTables < ActiveRecord::Migration

          def self.up
            @replace_me
    END

    t = self.parsed_tables
    linkers = self.parse_linking_tables

    rr = ''
    t.each_index do |i|
      rr += self.build_migration(t[i])
    end
    rr += linkers[0]
    r.gsub!("@replace_me", "\n"+rr)

    rdown=<<-END
          end #self.up




          def self.down
            @replace_me
          end #self.down

       end
    END

    rr = ''
    ind="            "
    t.each_index do |i|
      rr = ind + "drop_table :#{t[i][0][1]}\n" + rr
    end
    rr += linkers[1]

    rdown.gsub!("@replace_me", "\n"+rr)
	@migrations = r + rdown
    return @migrations

  end


  #Input:  table_template in form of arrays
  def self.build_migration(table_template)
    tt = table_template
    plural = tt[0][1]
    ind="            "
    r=ind + "create_table '#{plural}' do |t|\n"

    tt.each_index do |i|
      next if i == 0
      e = tt[i]
      default = :"#{e[0]}"
      default = @field_default[default].to_s
      default = ',   ' + default if default != ''

	  if e.size > 2
		#in this case, prepare parameters to replace default:
		default = ', '
		e.each_with_index do |par,i|
			default += ' '+par if i>1
		end
      end

      r += ind + '  t.' + e[0].to_s + "  '" + e[1].to_s + "'  "+ default + "\n"
    end
    r += ind + "end\n\n"
  end
  #--------------------------------------------------------------
  # migrations
  #==============================================================




  #============================================================
  # Routes
  #------------------------------------------------------------
  def self.add_relations_to_route_resources()
	@relations.each do |masterParentName, children|
		@routes_resources[masterParentName] ||= '';
		children.each do |childMasterName, child|
			child_plural = @models[childMasterName][:plural]
			@routes_resources[masterParentName] += "\t\tresources :"+child_plural + "\n"
		end
	end
  end

  def self.finish_routes_resources
	@routes_resources ||= {}
	@parsed_tables.each do |t|
		single=t[0][0]
		plural=t[0][1]
		if @routes_resources[single] 
			@routes_resources[single]="\tresources :#{plural} do\n" +
				@routes_resources[single]+
				"\tend\n"
		else
			@routes_resources[single]="\tresources :#{plural}\n"
		end 
	end	
  end

  def self.build_routes(application_name='test', login=true)
	return @routes if @routes
	@routes="#{application_name.camelize}::Application.routes.draw do\n"

	if login
		w =<<-ROUTES
			root	to: 'login#index'
			#match ''		=> 'login#index' 
			match 'login'	=> 'login#index'
			match 'logout'	=> 'login#logout'
			#get 'login/index'
		ROUTES
		@routes << w
	end

    if @routes_resources
		@routes_resources.each do |k,s|
			@routes << s
		end
	end	

	@routes << 'end'
	#  resources :users
  end

  def self.save_routes(root_to_save='.')
	root_to_save += '/config/'
	FileLib.save root_to_save+'routes.rb', @routes
  end
  #------------------------------------------------------------
  # Routes
  #============================================================





  #============================================================
  # Scaffolds
  #------------------------------------------------------------
  def self.build_scaffolds
	return 	@scaffolds if 	@scaffolds
	puts 'building scaffolds ...'
    t = self.parsed_tables
    r = []
    t.each_index do |i|
      r << self.build_scaffold(t[i])
    end
	@scaffolds=r
  end

  def self.build_scaffold(table_template)
	tt = table_template
    singular = tt[0][0]
	puts '  scaffold ' +singular + '...'   
    s = "rails generate scaffold " + singular
    tt.each_index do |i|
      next if i == 0
      e = tt[i]
      default = :"#{e[0]}"
      default = @field_default[default].to_s
      default = ',   ' + default if default != ''
      s += " " + e[1].to_s + ":" + e[0].to_s
    end
    return s
  end
  #============================================================




  #======================================================================
  # Build Models
  #----------------------------------------------------------------------

  #Input	if root_to_save is provided, will be saved there:
  def self.build_model_definition(table_template, root_to_save=nil)
    tt = table_template
    singular_name = tt[0][0]

	pts=self.parsed_tables
	result = "class #{singular_name} < ActiveRecord::Base\n"

	#loop through table's metaschema lines:
	tt.each_with_index do |line,i|
		if i>0 and line[1].slice(-3,3)=='_id'
			#field is in format xxxxx_id
			#belongs_to suspect is found, add it
			suspect=line[1].slice(0, line[1].size-3)
			found = false
			target_table_of_relation = nil
			pts.each do |pt|
				if pt[0][0].decamelize == suspect
					target_table_of_relation = pt[0][0] 
					result << '	belongs_to :'+suspect+"\n"
					break
				end
			end
		end
	end

	w=@has_and_belongs_to_many[singular_name]
	result << w if w

	if @relations[singular_name]
		#@relations[singular_name].dig 'for parent '+ singular_name
		@relations[singular_name].each do |i,relation|
			w=relation[:for_model]
			result << "\t#{w}\n"
		end
	end


	result << "end\n"
  end

  #Depends on previous methods, must be a last one:
  def self.build_model_definitions
	return @model_definitions if @model_definitions
	@model_definitions = {}
	self.parsed_tables.each do |pt|
		@model_definitions[pt[0][0]] = self.build_model_definition(pt)
	end
	@model_definitions
  end

  #Depends on previous methods, must be a last one:
  def self.save_models(root_to_save='.')
	root_to_save += '/app/models'
	FileUtils.mkdir_p root_to_save
	@model_definitions.each do |k,m|
		FileLib.save root_to_save + '/' + k.downcase + '.rb' ,m
	end
	@model_definitions
  end
  #----------------------------------------------------------------------	
  # Build Models
  #======================================================================
  	



  #======================================================================
  # Build Relational Partial Show and Form
  #----------------------------------------------------------------------	
  def self.build_relational_hforms_and_shows
	return {:relational_hforms=>@relational_hforms, :relational_hviews => @relational_hviews, :view_shows => @view_shows} if @relational_hforms
	@link_tables.each do |lt|
		[[lt[0],lt[1]],[lt[1],lt[0]]].each do |pair|
			pparent		=pair[0]
			cchild		=pair[1]
			cchildren	=@models[cchild][:plural_camel]
			children	=@models[cchild][:plural]
			parent		=@models[pparent][:decamelized]
			self.build_relational_view_and_show(pparent,cchild)
			if @view_shows[pparent]
				child_member = <<-EOT
		
					<b>#{cchildren}</b>
					<%= render @#{parent}.#{children} %>

					<b>Add an #{cchild}:</b>
					<%= render '#{children}/#{parent}_form' %>

					<!-- <?child members?> -->

				EOT
				@view_shows[pparent].gsub!('<!-- <?child members?> -->', child_member)
			end
		end
	end

	#@relations derived from <has_many> ... in table-meta-schema
	@relations.each do |parentMasterName, children|
		children.each do |childMasterName, child|
			pparent		=parentMasterName
			cchild		=childMasterName

			cchildren	=@models[cchild][:plural_camel]
			children	=@models[cchild][:plural]
			parent		=@models[pparent][:decamelized]
			self.build_relational_view_and_show(pparent,cchild)
			if @view_shows[pparent] #why to ask?
				child_member = <<-EOT
		
					<b>#{cchildren}</b>
					<%= render @#{parent}.#{children} %>

					<b>Add an #{cchild}:</b>
					<%= render '#{children}/#{parent}_form' %>

					<!-- <?child members?> -->

				EOT
				@view_shows[pparent].gsub!('<!-- <?child members?> -->', child_member)
			end
		end #children.each do |child_name, child|
	end #@relations.each

	{:relational_hforms=>@relational_hforms, :relational_hviews => @relational_hviews, :view_shows => @view_shows}

  end
	
  #does both views and shows:
  #adds "stuff" to "instance" variables: @relational_hforms,
  #									     @relational_hviews

  def self.build_relational_view_and_show(master,child)
	@relational_hforms ||={}
	@relational_hforms[master] ||= {}

	@relational_hviews ||={}
	@relational_hviews[master] ||= {}

	c_model=@models[child]

	mt=@models[master][:table]
	ct=@models[child][:table]

	m_s=@models[master][:decamelized]
	m_p=@models[master][:plural] 
	c_p=c_model[:plural]
	c_s=c_model[:decamelized]
	c_c=child

	template ="\t\t<%= form_for([@#{m_s}, @#{m_s}.#{c_p}.build]) do |f| %>\n\t\t\t<table><tr>\n"

	show_template ="\t\t<p id=\"notice\"><%= notice %></p>\n\t\t\t<table><tr>\n"
	field_template =<<-EOT
				<td class="field">
					<%= f.label :(FIELD) %><br />
					<%= f.(TYPE) :(FIELD) %>
				</td>
	EOT
	show_field_template =<<-EOT
				<td>
					<?Field?>:
					<%= <?parent?>.<?field?> %>
				</td>
	EOT

	ct.each_with_index do |c,i|
		next if i == 0
		next if c[1] == m_s+'_id' #skip reference to collection owner
		tmp=''
		show=''
		case c[0]
			when	'string'
					tmp = field_template.gsub('(TYPE)','text_field')
			when	'text'
					tmp = field_template.sub('(TYPE)','text_area')
			when	'boolean'
					tmp = field_template.sub('(TYPE)','check_box')
			when	'datetime'
					tmp = field_template.sub('(TYPE)','datetime_select')
			when	'integer'
					tmp = field_template.sub('(TYPE)','number_field')
		end
		template << tmp.gsub('(FIELD)',c[1])
		show = show_field_template.gsub('<?Field?>',c[1].camelize) #TODm titalize: Title of field.
		show.gsub!('<?parent?>',	c_model[:decamelized])
		show.gsub!('<?field?>',		c[1])

		show_template << show
	end
	
	@relational_hforms[master][child] =<<-SUBMIT
				\n#{template}
			</tr>
			<tr>
				<td class="actions">
					<%= f.submit %>
				</td>
			</tr></table>	
		<% end %>
	SUBMIT

	#TODm this is an apparent bug:
	#link_to 'Destroy depneds on both parent and child ... then file is save by child name
	#if there are to parents then first-parent info is erased by saving a file:
	@relational_hviews[master][child] =<<-EOT
				\n#{show_template}
				</tr>
				<tr>
					<% if @#{m_s} %>
						<td><%= link_to 'Destroy #{c_c}', [ @#{m_s}, #{c_s}], 	# [#{c_s}.#{m_s}, #{c_s}],
					               confirm:	'Are you sure?',
					               method:	:delete %>
						</td>
					<% end %>
				</tr></table>	
	EOT
  end


  #Input:	usually: root_to_save = is app root 
  def self.save_relational_hforms_and_shows(root_to_save='.')
	root_to_save += '/app/views'
	@relational_hforms.each do |parent,children|
		p_single=parent.decamelize
		children.each do |child,template|
			c_model=@models[child]
			folder=root_to_save + '/' + 	c_model[:plural]
			FileUtils.mkdir_p folder
			FileLib.save folder + '/_'+p_single + '_form.html.erb',template
		end
	end
	@relational_hviews.each do |parent,children|
		p_single=parent.decamelize
		children.each do |child,template|
			c_model=@models[child]
			folder=root_to_save + '/' + 	c_model[:plural]
			FileUtils.mkdir_p folder
			FileLib.save folder + '/_'+c_model[:decamelized] + '.html.erb',template
		end
	end
  end

  #----------------------------------------------------------------------	
  # Build Relational Partial Show and Form
  #======================================================================




  #======================================================================
  # Build Show View
  #----------------------------------------------------------------------	
  def self.build_view_shows
	return @view_shows if @view_shows
	show_field_template =<<-EOT
		<p>
		  <b><?Field?>:</b>
		  <%= @<?model?>.<?field?> %>
		</p>
	EOT

	footer_template =<<-EOT

		<!-- <?child members?> -->

		<%= link_to 'Edit', edit_<?model?>_path(@<?model?>) %> |
		<%= link_to 'List', <?models?>_path %>

	EOT

	@view_shows = {}

	@models.each do |camel_name, model|
		show_template ="\t\t<p id=\"notice\"><%= notice %></p>\n"
		model[:table].each_with_index do |field,i|
			next if i == 0
			show = show_field_template.gsub('<?Field?>',field[1].camelize)
			show.gsub!('<?model?>',	model[:decamelized])
			show.gsub!('<?field?>',	field[1])
			show_template << show
		end

		footer=footer_template.gsub('<?model?>',model[:decamelized])
		footer.gsub!('<?models?>',model[:plural])

		@view_shows[camel_name] = show_template + footer
	end
	@view_shows
  end

  def self.save_view_shows(root_to_save='.')
	root_to_save += '/app/views'
	@view_shows.each do |camel_name,view|
		model=@models[camel_name]
		folder=root_to_save + '/' + model[:plural]
		FileUtils.mkdir_p folder
		FileLib.save folder + '/show.html.erb',view
	end
  end
  #----------------------------------------------------------------------	
  # Build Show View
  #======================================================================














  #======================================================================
  # Model Controllers
  #----------------------------------------------------------------------	
  def self.build_model_controllers
	return @controllers if @controllers
	@controllers={}
	c_template			=File.read File.expand_path('../templates/app/controllers/controller.rb', __FILE__)
	a_template			=File.read File.expand_path('../templates/app/controllers/controller_association.rb', __FILE__)
	destroy_a_template	=File.read File.expand_path('../templates/app/controllers/controller_delete_association.rb', __FILE__)

	@models.each do |name,model|
		controller=c_template.gsub('<%PluralOwner%>',model[:plural_camel])
		controller.gsub!('<%Owner%>',model[:name])
		controller.gsub!('<%decamelized_owner%>',model[:decamelized])
		controller.gsub!('<%plural_owner%>',model[:plural])

		#TODm derived from "views" ... "backward" ... must make "relations" and derive "all" from there:
		#@relational_hforms[master][child]=template

		@models.each do |child,cview|
		  if @relational_hforms[child]
			@relational_hforms[child].each do |nname,view|
				next if nname != name
				c_model=@models[child]

				ass=a_template.gsub('<%decamelized_owner%>',model[:decamelized])
				ass.gsub!('<%plural_owner%>',model[:plural])
				ass.gsub!('<%Owner%>',model[:name])

				ass.gsub!('<%decamelized_child%>',c_model[:decamelized])
				ass.gsub!('<%Child%>',c_model[:name])
				ass.gsub!('<%plural_child%>',c_model[:plural])

				controller.gsub!(
						'#<%template_of_controller_association.rb%>',
						'#<%template_of_controller_association.rb%>'+"\n"+
						ass
				)

				destroy_ass=destroy_a_template.gsub('<%decamelized_owner%>',model[:decamelized])
				destroy_ass.gsub!('<%plural_owner%>',model[:plural])

				destroy_ass.gsub!('<%decamelized_child%>',c_model[:decamelized])
				destroy_ass.gsub!('<%Child%>',c_model[:name])
				destroy_ass.gsub!('<%plural_child%>',c_model[:plural])

				controller.gsub!(
						'#<%template_of_destroy_association.rb%>',
						'#<%template_of_destroy_association.rb%>'+"\n"+
						destroy_ass
				)
			end #@relational_hforms[child].each do |nname,view|
		  end #if @relational_hforms[child]
		end #@models.each do |child,cview|
		@controllers[name]=controller
	end
	@controllers
  end

  def self.save_model_controllers(root_to_save='.')
	folder = root_to_save + '/app/controllers'
	FileUtils.mkdir_p folder
	@controllers.each do |parent,controller|
		plural=@models[parent][:plural]
		FileLib.save folder + '/'+plural + '_controller.rb',controller
	end
  end
  #----------------------------------------------------------------------	
  # Model Controllers
  #======================================================================



  #======================================================================
  # Footnotes
  #----------------------------------------------------------------------	
  def self.build_shared_footnotes
		return @shared_footnotes if @shared_footnotes
		@shared_footnotes=File.read File.expand_path('../templates/app/views/shared/_footer.html.erb',__FILE__)
		model_list_template =
			"\t<span class=\"foot_nav\"><%= link_to '<?mmodels?>', <?models?>_path %></span>\n"
		seed = "<!-- <?model list?> -->\n"

		@models.each do |mmodel,m|
			model_list = model_list_template.gsub('<?mmodels?>',m[:plural_camel])
			model_list.gsub!('<?models?>',m[:plural])
			if mmodel=='User'
				model_list =	"	<% if session[:role]=='admin' %>\n"+
										"\t"+model_list +
								"	<% end %>\n"
			end
			@shared_footnotes.gsub! seed, model_list+seed
		end
		@shared_footnotes
  end
  def self.save_shared_footnotes(root_to_save='.')
	folder = root_to_save + '/app/views/shared'
	FileUtils.mkdir_p folder
	@controllers.each do |parent,controller|
		FileLib.save folder + '/_footer.html.erb',@shared_footnotes
	end
  end
  #----------------------------------------------------------------------	
  # Footnotes
  #======================================================================

  #======================================================================
  # Login
  #----------------------------------------------------------------------	
  def self.build_login
		return @login if @login
		@login=File.read File.expand_path('../templates/app/views/login/index.html.erb',__FILE__)
  end
  def self.save_login(root_to_save='.')
	folder = root_to_save + '/app/views/login'
	FileUtils.mkdir_p folder
	FileLib.save folder + '/index.html.erb',@login
  end
  #----------------------------------------------------------------------	
  # Login
  #======================================================================

end #class GenerateMVC





